"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateRef = exports.generateEnum = exports.generateTable = void 0;
const index_1 = require("../../../../index");
const generateTable = (table, nameEnums) => {
    let tableDoc = `Table ${table.name} {\n`;
    const data = table.columns
        .reduce((target, item) => {
        var _a;
        // name column
        target += `\t${item.name} `;
        let type = [String, Number, Boolean].includes(item.type)
            ? item.type.name.toLocaleLowerCase()
            : [index_1.ObjectId].includes(item.type)
                ? "string"
                : "json";
        // type column
        if (item.isEnum) {
            const enumName = (_a = nameEnums.find(e => { var _a; return e.enum === ((_a = item.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType); })) === null || _a === void 0 ? void 0 : _a.name;
            target += `${enumName} `;
        }
        else {
            target += item.isPrimaryKey ? `string(30) ` : `${type} `;
        }
        //option
        let option = '';
        if (item.isPrimaryKey) {
            option = `[pk, not null ]`;
        }
        else {
            if (item.isRequire) {
                option += `not null, `;
            }
            if (item.unique) {
                option += `unique, `;
            }
            if (item.default !== undefined) {
                option += `default: ${typeof item.default == "string" || typeof item.default == "object"
                    ? `"${item.default.toString()}"`
                    : `${item.default}`}, `;
            }
            option = `[${option.trim().replace(/^,|,$/g, '')}]`;
        }
        option = option === "[]" ? "" : option;
        target += `${option} \n`;
        return target;
    }, "");
    tableDoc += data;
    tableDoc += `}\n\n`;
    return tableDoc;
};
exports.generateTable = generateTable;
const generateEnum = (nameEnums) => {
    return nameEnums.reduce((target, item) => {
        const enumTable = `Enum ${item.name} {`;
        const valueEnums = Object.values(item.enum).reduce((targetEnum, valueEnum) => {
            targetEnum += `\t${valueEnum}\n`;
            return targetEnum;
        }, "");
        target += `${enumTable}\n${valueEnums}}\n`;
        return target;
    }, "");
};
exports.generateEnum = generateEnum;
const generateRef = (relations) => {
    const refs = relations.reduce((target, item) => {
        // Ref: posts.user_id > users.id;
        const nameTableChild = (0, index_1.getMetadataTable)().getTable(item.refs.modelChild()).name;
        const nameTableParent = (0, index_1.getMetadataTable)().getTable(item.refs.modelParent()).name;
        const primaryKey = (0, index_1.getMetadataTable)().getColumns(item.refs.modelParent()).find(e => e.isPrimaryKey);
        const ref = `Ref: ${nameTableChild}.${item.refs.FK.toString()} > ${nameTableParent}.${primaryKey ? primaryKey.name : "id"}\n`;
        if (!target.includes(ref)) {
            target.push(ref);
        }
        return target;
    }, []);
    return refs.join('');
};
exports.generateRef = generateRef;
