"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.connectDatabaseSql = void 0;
const metadata_1 = require("../../global/metadata");
const commons_1 = require("../../commons");
const sequelize_1 = require("sequelize");
const decorators_1 = require("../../decorators");
const injector_1 = require("../../di/injector");
const metadata_2 = require("../../metadata");
const generateSchemaDefinition = (table) => {
    let schemaDefinition = {};
    for (const field of table.columns) {
        schemaDefinition[field.name] = {};
        //-- check type
        switch (field.type) {
            case Number:
                if (field.isFloat) {
                    schemaDefinition[field.name]["type"] = sequelize_1.DataTypes.FLOAT;
                }
                else {
                    if (field.isBigInt) {
                        schemaDefinition[field.name]["type"] = sequelize_1.DataTypes.BIGINT;
                    }
                    else {
                        schemaDefinition[field.name]["type"] = sequelize_1.DataTypes.INTEGER;
                    }
                }
                break;
            case String:
                schemaDefinition[field.name]["type"] = sequelize_1.DataTypes.STRING;
                break;
            case Boolean:
                schemaDefinition[field.name]["type"] = sequelize_1.DataTypes.BOOLEAN;
                break;
            case Array:
            default:
                schemaDefinition[field.name]["type"] = sequelize_1.DataTypes.TEXT;
                break;
        }
        //-- check primaryKey
        if (field.isPrimaryKey) {
            schemaDefinition[field.name]["primaryKey"] = true;
            schemaDefinition[field.name]["autoIncrement"] = true;
        }
        //-- check default value
        if (field.default !== undefined) {
            schemaDefinition[field.name]["defaultValue"] = field.default;
        }
        //-- check nullable
        schemaDefinition[field.name]["allowNull"] = !field.isRequire;
    }
    return schemaDefinition;
};
const connectDatabaseSql = (config, isTest) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        let collections = {};
        const sequelize = new sequelize_1.Sequelize({
            dialect: config.type,
            database: config.database,
            username: config.username,
            password: config.password,
            host: config.host,
            port: config.port,
            logging: (message) => {
                if (config.isLog || config.isLog == undefined) {
                    console.log(message);
                }
            },
        });
        (0, metadata_1.getMetadataTable)().getTables().forEach((table) => {
            const schemaDefinition = generateSchemaDefinition(table);
            collections[table.name] = sequelize.define(table.name, schemaDefinition, {
                tableName: table.name,
                freezeTableName: true,
                name: {
                    singular: table.name,
                    plural: table.name,
                },
                timestamps: false,
            });
        });
        (0, metadata_1.getMetadataTable)().getTables().forEach((table) => {
            const schema = collections[table.name];
            table.relations.forEach(relation => {
                const tableInclude = (0, metadata_1.getMetadataTable)().getTable(relation.type == decorators_1.TypeRelation.BELONG_TO
                    ? relation.refs.modelParent()
                    : relation.refs.modelChild());
                switch (relation.type) {
                    case decorators_1.TypeRelation.BELONG_TO:
                        schema.belongsTo(collections[tableInclude.name], {
                            foreignKey: relation.refs.FK.toString(),
                            as: relation.fieldName
                        });
                        break;
                    case decorators_1.TypeRelation.HAS_MANY:
                        schema.hasMany(collections[tableInclude.name], {
                            foreignKey: relation.refs.FK.toString(),
                            as: relation.fieldName
                        });
                        break;
                    case decorators_1.TypeRelation.HAS_ONE:
                        schema.hasOne(collections[tableInclude.name], {
                            foreignKey: relation.refs.FK.toString(),
                            as: relation.fieldName
                        });
                        break;
                    default:
                        break;
                }
            });
        });
        injector_1.injector.get(metadata_2.Metadata).setDbSqlCollections(collections);
        yield sequelize.authenticate();
        if (!isTest) {
            console.log("****************************");
            console.log("*    Starting Server");
            console.log(`*    ${(0, commons_1.csuccess)(`DB:${config.database}`)}`);
            console.log(`*    ${(0, commons_1.csuccess)(`Type: ${config.type}`)}`);
            console.log(`*    ${(0, commons_1.csuccess)("DB Connection: OK")}`);
            console.log(`****************************`);
        }
        return sequelize;
    }
    catch (error) {
        console.log(`*    ${(0, commons_1.cerror)(`Error connecting to DB: ${error}\n****************************`)}\n`);
        process.exit();
    }
});
exports.connectDatabaseSql = connectDatabaseSql;
