import { Collection, ObjectId } from "mongodb";
import { IListCollection } from "../connections/mongodb";
import { QueryCondition, FindOptions, KeyColumns, ModelQuery } from "./interfaces/QueryCondition";
import { IColumn } from "../decorators/interfaces/table";
import { IPagination } from "../global/interface";
import { WithRelation } from "./interfaces/WithRelation";
import { DBConnection } from "../connections/types";
import { IQueryBuilder } from "./interfaces/IQueryBuilder";
export declare abstract class QueryBuilder<T> {
    protected connection?: DBConnection;
    protected db?: IQueryBuilder<T>;
    protected collection?: Collection;
    protected getTableName(): keyof IListCollection;
    getColumns(): IColumn[];
    getNameColumns(): KeyColumns<T>;
    getRelations(): import("../..").IRelationship<any, any>[];
    protected getPrimaryKey(): IColumn;
    protected getDB(): IQueryBuilder<T>;
    with<K>(params: WithRelation<T, K> | WithRelation<T, K>[]): this;
    select(columns: KeyColumns<T>): this;
    findById(id: string | ObjectId): Promise<ModelQuery<T> | null>;
    findOne(filter: QueryCondition<T>): Promise<ModelQuery<T> | null>;
    find(filter: QueryCondition<ModelQuery<T>>, options?: FindOptions<ModelQuery<T>>): Promise<ModelQuery<T>[]>;
    paginate(filter: QueryCondition<ModelQuery<T>>, paginate: IPagination, options?: FindOptions<ModelQuery<T>>): Promise<{
        data: ModelQuery<T>[];
        totalCount: number;
    }>;
    count(filter: QueryCondition<ModelQuery<T>>): Promise<number>;
    createOne(item: Partial<ModelQuery<T>>): Promise<ModelQuery<T>>;
    updateMany(filter: QueryCondition<ModelQuery<T>>, item: Partial<ModelQuery<T>>): Promise<ModelQuery<T>[]>;
    delete(filter: QueryCondition<ModelQuery<T>>): Promise<any>;
    deleteMany(filter: QueryCondition<ModelQuery<T>>): Promise<any>;
    private validateParamBeforeSaveDB;
}
