import { Db, ObjectId } from "mongodb";
import { QueryCondition, FindOptions, ModelQuery, KeyColumns, Op, OpSpecial, OpArray } from "./interfaces/QueryCondition";
import { WithRelation } from "./interfaces/WithRelation";
import { IQueryBuilder } from "./interfaces/IQueryBuilder";
import { Constructor, Table } from "../decorators";
import { IPagination } from "../global";
import { QueryBuilder } from ".";
export declare class MongoQueryBuilder<T> implements IQueryBuilder<T> {
    readonly db: Db;
    readonly table: Table;
    withRelations: WithRelation<T, any>[];
    selectColumns: KeyColumns<T>;
    constructor(db: Db, table: Table);
    protected getPrimaryKey(): import("../decorators").IColumn;
    private getCollection;
    generateWith<K>(params: WithRelation<T, K> | WithRelation<T, K>[], model: Constructor<QueryBuilder<T>>): WithRelation<T, any>[];
    with<K>(params: WithRelation<T, K> | WithRelation<T, K>[], model: QueryBuilder<T>): void;
    generateAggregate(withRelations: WithRelation<T, any>[], isFromCRUD: boolean): any[];
    findByPk(pk: ObjectId | string | number): Promise<ModelQuery<T> | null>;
    find(filter: QueryCondition<ModelQuery<T>>, options?: FindOptions<ModelQuery<T>>): Promise<ModelQuery<T>[]>;
    paginate(filter: QueryCondition<ModelQuery<T>>, paginate: IPagination, options?: FindOptions<ModelQuery<T>>): Promise<{
        data: ModelQuery<T>[];
        totalCount: number;
    }>;
    count(filter: QueryCondition<ModelQuery<T>>): Promise<number>;
    createOne(item: Partial<ModelQuery<T>>): Promise<ModelQuery<T>>;
    updateMany(filter: QueryCondition<ModelQuery<T>>, item: Partial<ModelQuery<T>>): Promise<import("mongodb").UpdateResult<import("bson").Document>>;
    deleteOne(filter: QueryCondition<ModelQuery<T>>): Promise<import("mongodb").DeleteResult>;
    deleteMany(filter: QueryCondition<ModelQuery<T>>): Promise<import("mongodb").DeleteResult>;
    select(selectColumns: KeyColumns<T>): void;
    reset(): void;
    mapToMongoOperator(op: Op | OpSpecial | OpArray, attribute: keyof ModelQuery<T>, value: T[keyof ModelQuery<T>]): {
        [x: string]: {
            $regex: T[Exclude<keyof T, import("./interfaces/QueryCondition").FunctionQueryBuilder>];
            $options: string;
        };
    } | {
        [x: string]: {
            [x: string]: T[Exclude<keyof T, import("./interfaces/QueryCondition").FunctionQueryBuilder>];
        };
    };
    getQueryObjectFromQueryConditions(queries: QueryCondition<T>): QueryCondition<T>;
}
