"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryRequest = void 0;
const mongodb_1 = require("mongodb");
const errorBuilder_1 = require("../errorBuilder");
const type_1 = require("../helpers/type");
class QueryRequest {
    constructor(queryBuilder) {
        this.queryBuilder = queryBuilder;
    }
    convertQueriesIncludeObjectId(key, queryParse, resultQueries) {
        const queryColumn = queryParse[key];
        if (Array.isArray(queryColumn)) {
            resultQueries[key] = {
                $in: queryColumn.map((value) => {
                    if (typeof value != "string") {
                        throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${key.toString()} not allow`);
                    }
                    return (0, type_1.objectId)(value);
                }),
            };
        }
        else {
            throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${key.toString()} not allow`);
        }
        return resultQueries;
    }
    convertQueriesIncludeString(key, queryParse, resultQueries) {
        const queryColumn = queryParse[key];
        if (typeof queryColumn == "string") {
            resultQueries[key] = queryColumn;
        }
        else if (typeof queryColumn == "object") {
            const { value, isAbsolute } = queryColumn;
            if (typeof value != "string" || (isAbsolute !== undefined && typeof isAbsolute != "boolean")) {
                throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${key.toString()} not allow`);
            }
            if (isAbsolute) {
                resultQueries[key] = value;
            }
            else {
                resultQueries[key] = { $regex: value, $options: "i" };
            }
        }
        else {
            throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${key.toString()} not allow`);
        }
        return resultQueries;
    }
    convertQueriesIncludeNumber(key, queryParse, resultQueries) {
        const queryColumn = queryParse[key];
        if (Array.isArray(queryColumn)) {
            resultQueries[key] = {
                $in: queryColumn,
            };
        }
        else if (typeof queryColumn == "object") {
            const { isMaxExclusive, isMinExclusive, max, min } = queryColumn;
            const tmpQueriesNumber = {};
            if (max !== undefined) {
                isMaxExclusive
                    ? Object.assign(tmpQueriesNumber, { $lt: max })
                    : Object.assign(tmpQueriesNumber, { $lte: max });
            }
            if (min !== undefined) {
                isMinExclusive
                    ? Object.assign(tmpQueriesNumber, { $gt: min })
                    : Object.assign(tmpQueriesNumber, { $gte: min });
            }
            resultQueries[key] = tmpQueriesNumber;
        }
        else {
            throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${key.toString()} not allow`);
        }
        return resultQueries;
    }
    /*================================+
       |CHECK VALIDATOR QUERY REQUEST   |
       +================================*/
    getQueries(queries) {
        try {
            let resultQueries = {};
            if (!queries)
                return {};
            const queryParse = JSON.parse(queries);
            if (!queryParse) {
                throw new Error("Condition fail");
            }
            const columns = this.queryBuilder.getColumns();
            const checkKeyExist = Object.keys(queryParse).filter((key) => {
                return !columns.map(e => e.name).includes(key);
            });
            if (checkKeyExist.length) {
                throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${checkKeyExist} not allow`);
            }
            Object.keys(queryParse).forEach((key) => {
                const column = columns.find((x) => x.name == key);
                if (column) {
                    switch (column.type) {
                        case mongodb_1.ObjectId:
                            resultQueries = this.convertQueriesIncludeObjectId(key, queryParse, resultQueries);
                            break;
                        case String:
                            resultQueries = this.convertQueriesIncludeString(key, queryParse, resultQueries);
                            break;
                        case Number:
                            resultQueries = this.convertQueriesIncludeNumber(key, queryParse, resultQueries);
                        default:
                            break;
                    }
                }
                else {
                    throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${key} not allow`);
                }
            });
            return resultQueries;
        }
        catch (error) {
            throw error;
        }
    }
    validateAndConverOperatorFilter(queries) {
        Object.keys(queries).forEach(keyObj => {
            if (keyObj == '$and' || keyObj == '$or') {
                return queries[keyObj].map(obj => this.validateAndConverOperatorFilter(obj));
            }
            else {
                const operator = Object.keys(queries[keyObj])[0];
                const field = keyObj;
                const value = queries[field][operator];
                const columns = this.queryBuilder.getColumns();
                const column = columns.find((x) => x.name == field);
                if (!column) {
                    throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                }
                if (!((operator === "$eq" || operator === "$ne") && value === null)) {
                    switch (column.type) {
                        case mongodb_1.ObjectId:
                            try {
                                if (operator == '$in' || operator == '$nin') {
                                    if (!Array.isArray(value)) {
                                        throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                    }
                                    value.forEach(e => {
                                        if (typeof e != 'string') {
                                            throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                        }
                                    });
                                    queries[keyObj][operator] = value.map(e => (0, type_1.objectId)(e));
                                }
                                else {
                                    queries[keyObj][operator] = (0, type_1.objectId)(value);
                                }
                            }
                            catch (error) {
                                throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                            }
                            break;
                        case String:
                            if (operator == '$in' || operator == '$nin') {
                                if (!Array.isArray(value)) {
                                    throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                }
                                value.forEach(e => {
                                    if (typeof e != 'string') {
                                        throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                    }
                                });
                            }
                            else {
                                if (typeof value !== 'string') {
                                    throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                }
                            }
                            break;
                        case Number:
                            if (operator == '$in' || operator == '$nin') {
                                if (!Array.isArray(value)) {
                                    throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                }
                                value.forEach(e => {
                                    if (typeof e !== 'number') {
                                        throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                    }
                                });
                            }
                            else {
                                if (typeof value !== 'number') {
                                    throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                                }
                            }
                            break;
                        case Boolean:
                            if (typeof value !== 'boolean') {
                                throw errorBuilder_1.ErrorBuilder.validator(`Key or type ${field} not allow`);
                            }
                        default:
                            break;
                    }
                }
            }
        });
        return queries;
    }
}
exports.QueryRequest = QueryRequest;
